require 'clamp'

autoload :Issue,       'octokom/command/issue'
autoload :PullRequest, 'octokom/command/pull_request'

module Octokom
  class Command < Clamp::Command
    include Octokom::Shell

    # Shortcut for `Clamp::Command.option` with `:flag` argument.
    #
    # For example:
    #   flag ['-a'], A option'
    #
    # Is the same as:
    #   option ['-a'], :flag, 'A option'
    def self.flag(tags, desc, &block)
      option_without_type(tags, :flag, desc, &block)
    end

    # Alias for the `Clamp::Command.option` method that uses the long option
    # tag as type string.
    #
    # For example:
    #   Octokom::Command.option ['-a', '--abc'], 'ABC option'
    #
    # Is the same as:
    #   Clamp::Command.option ['-a', '--abc'], 'ABC', 'ABC option'
    def self.option_with_type(tags, desc, &block)
      type = tags.first[/^--(.*)$/, 1].upcase
      option_without_type(tags, type, desc, &block)
    end

    # Override `Clamp::Command.execute` method so it can be used as a block.
    def self.execute(&block)
      define_method(:execute, block) do
        yield if block_given?
      end
    end

    # Create aliases for the Clamp::Command `option` and `subcommand` methods.
    singleton_class.class_eval do
      alias_method :option_without_type, :option
      alias_method :option, :option_with_type
      alias_method :sub_command, :subcommand
    end

    flag ['--dry-run', '-y'], 'Do nothing, only show what would happen.'
    flag ['--verbose', '-v'], 'Show the debug output.'

    flag ['--version'], 'Version number.' do
      puts "Octokom #{Octokom::VERSION}"
    end

    # Override the Clamp help message option.
    flag ['--help'], 'Help message.' do
      request_help
    end

    # Initializes a new Repository instance that is available in all
    # Octokom::Command instances.
    def git
      @git ||= Octokom::Repository.new(verbose?)
    end

    def authenticate(&block)
      yield Octokom::Client.authenticate
    end
  end
end
